@@ -564,6 +564,9 @@ struct dr_ste_htbl *dr_ste_htbl_alloc(struct dr_icm_pool *pool,
 		atomic_init(&ste->refcount, 0);
 		list_node_init(&ste->miss_list_node);
 		list_head_init(&htbl->miss_list[i]);
+		ste->next_htbl = NULL;
+		ste->rule_rx_tx = NULL;
+		ste->ste_chain_location = 0;
 	}
 
 	htbl->chunk_size = chunk_size;
@@ -591,7 +594,8 @@ void dr_ste_set_actions_tx(struct dr_ste_ctx *ste_ctx,
 			   struct dr_ste_actions_attr *attr,
 			   uint32_t *added_stes)
 {
-	ste_ctx->set_actions_tx(action_type_set, hw_ste_arr, attr, added_stes);
+	ste_ctx->set_actions_tx(action_type_set, ste_ctx->actions_caps,
+				hw_ste_arr, attr, added_stes);
 }
 
 void dr_ste_set_actions_rx(struct dr_ste_ctx *ste_ctx,
@@ -600,7 +604,8 @@ void dr_ste_set_actions_rx(struct dr_ste_ctx *ste_ctx,
 			   struct dr_ste_actions_attr *attr,
 			   uint32_t *added_stes)
 {
-	ste_ctx->set_actions_rx(action_type_set, hw_ste_arr, attr, added_stes);
+	ste_ctx->set_actions_rx(action_type_set, ste_ctx->actions_caps,
+				hw_ste_arr, attr, added_stes);
 }
 
 const struct dr_ste_action_modify_field *
@@ -608,8 +613,7 @@ dr_ste_conv_modify_hdr_sw_field(struct dr_ste_ctx *ste_ctx,
 				struct dr_devx_caps *caps,
 				uint16_t sw_field)
 {
-	return ste_ctx->get_action_hw_field(sw_field, caps);
-
+	return ste_ctx->get_action_hw_field(ste_ctx, sw_field, caps);
 }
 
 void dr_ste_set_action_set(struct dr_ste_ctx *ste_ctx,
@@ -663,6 +667,60 @@ int dr_ste_set_action_decap_l3_list(struct dr_ste_ctx *ste_ctx,
 						 used_hw_action_num);
 }
 
+static int dr_ste_alloc_modify_hdr_chunk(struct mlx5dv_dr_action *action,
+					 uint32_t chunck_size)
+{
+	int ret;
+
+	action->rewrite.param.chunk = dr_icm_alloc_chunk(action->rewrite.dmn->action_icm_pool,
+							 chunck_size);
+	if (!action->rewrite.param.chunk)
+		return ENOMEM;
+
+	action->rewrite.param.index = (action->rewrite.param.chunk->icm_addr -
+		action->rewrite.dmn->info.caps.hdr_modify_icm_addr) /
+		ACTION_CACHE_LINE_SIZE;
+
+	ret = dr_send_postsend_action(action->rewrite.dmn, action);
+	if (ret)
+		goto free_chunk;
+
+	return 0;
+
+free_chunk:
+	dr_icm_free_chunk(action->rewrite.param.chunk);
+	return ret;
+}
+
+static void dr_dealloc_modify_hdr_chunk(struct mlx5dv_dr_action *action)
+{
+	dr_icm_free_chunk(action->rewrite.param.chunk);
+}
+
+int dr_ste_alloc_modify_hdr(struct mlx5dv_dr_action *action)
+{
+	uint32_t dynamic_chunck_size;
+
+	dynamic_chunck_size = ilog32(action->rewrite.param.num_of_actions - 1);
+
+	/* HW modify action index granularity is at least 64B */
+	dynamic_chunck_size = max_t(uint32_t, dynamic_chunck_size,
+				    DR_CHUNK_SIZE_8);
+
+	if (action->rewrite.dmn->modify_header_ptrn_mngr)
+		return action->rewrite.dmn->ste_ctx->alloc_modify_hdr_chunk(action, dynamic_chunck_size);
+
+	return dr_ste_alloc_modify_hdr_chunk(action, dynamic_chunck_size);
+}
+
+void dr_ste_free_modify_hdr(struct mlx5dv_dr_action *action)
+{
+	if (action->rewrite.dmn->modify_header_ptrn_mngr)
+		return action->rewrite.dmn->ste_ctx->dealloc_modify_hdr_chunk(action);
+
+	return dr_dealloc_modify_hdr_chunk(action);
+}
+
 static int dr_ste_build_pre_check_spec(struct mlx5dv_dr_domain *dmn,
 				       struct dr_match_spec *m_spec,
 				       struct dr_match_spec *v_spec)
@@ -761,336 +819,335 @@ int dr_ste_build_ste_arr(struct mlx5dv_dr_matcher *matcher,
 	return 0;
 }
 
-static void dr_ste_copy_mask_misc(char *mask, struct dr_match_misc *spec)
+static void dr_ste_copy_mask_misc(char *mask, struct dr_match_misc *spec, bool clear)
 {
-	spec->gre_c_present = DEVX_GET(dr_match_set_misc, mask, gre_c_present);
-	spec->gre_k_present = DEVX_GET(dr_match_set_misc, mask, gre_k_present);
-	spec->gre_s_present = DEVX_GET(dr_match_set_misc, mask, gre_s_present);
-	spec->source_vhca_port = DEVX_GET(dr_match_set_misc, mask, source_vhca_port);
-	spec->source_sqn = DEVX_GET(dr_match_set_misc, mask, source_sqn);
+	spec->gre_c_present = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_c_present, clear);
+	spec->gre_k_present = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_k_present, clear);
+	spec->gre_s_present = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_s_present, clear);
+	spec->source_vhca_port = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, source_vhca_port, clear);
+	spec->source_sqn = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, source_sqn, clear);
 
-	spec->source_port = DEVX_GET(dr_match_set_misc, mask, source_port);
+	spec->source_port = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, source_port, clear);
 
-	spec->outer_second_prio = DEVX_GET(dr_match_set_misc, mask, outer_second_prio);
-	spec->outer_second_cfi = DEVX_GET(dr_match_set_misc, mask, outer_second_cfi);
-	spec->outer_second_vid = DEVX_GET(dr_match_set_misc, mask, outer_second_vid);
-	spec->inner_second_prio = DEVX_GET(dr_match_set_misc, mask, inner_second_prio);
-	spec->inner_second_cfi = DEVX_GET(dr_match_set_misc, mask, inner_second_cfi);
-	spec->inner_second_vid = DEVX_GET(dr_match_set_misc, mask, inner_second_vid);
+	spec->outer_second_prio = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, outer_second_prio, clear);
+	spec->outer_second_cfi = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, outer_second_cfi, clear);
+	spec->outer_second_vid = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, outer_second_vid, clear);
+	spec->inner_second_prio = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, inner_second_prio, clear);
+	spec->inner_second_cfi = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, inner_second_cfi, clear);
+	spec->inner_second_vid = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, inner_second_vid, clear);
 
 	spec->outer_second_cvlan_tag =
-		DEVX_GET(dr_match_set_misc, mask, outer_second_cvlan_tag);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, outer_second_cvlan_tag, clear);
 	spec->inner_second_cvlan_tag =
-		DEVX_GET(dr_match_set_misc, mask, inner_second_cvlan_tag);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, inner_second_cvlan_tag, clear);
 	spec->outer_second_svlan_tag =
-		DEVX_GET(dr_match_set_misc, mask, outer_second_svlan_tag);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, outer_second_svlan_tag, clear);
 	spec->inner_second_svlan_tag =
-		DEVX_GET(dr_match_set_misc, mask, inner_second_svlan_tag);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, inner_second_svlan_tag, clear);
 
-	spec->gre_protocol = DEVX_GET(dr_match_set_misc, mask, gre_protocol);
+	spec->gre_protocol = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_protocol, clear);
 
-	spec->gre_key_h = DEVX_GET(dr_match_set_misc, mask, gre_key_h);
-	spec->gre_key_l = DEVX_GET(dr_match_set_misc, mask, gre_key_l);
+	spec->gre_key_h = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_key_h, clear);
+	spec->gre_key_l = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_key_l, clear);
 
-	spec->vxlan_vni = DEVX_GET(dr_match_set_misc, mask, vxlan_vni);
+	spec->vxlan_vni = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, vxlan_vni, clear);
 
-	spec->geneve_vni = DEVX_GET(dr_match_set_misc, mask, geneve_vni);
-	spec->geneve_oam = DEVX_GET(dr_match_set_misc, mask, geneve_oam);
+	spec->geneve_vni = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, geneve_vni, clear);
+	spec->geneve_oam = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, geneve_oam, clear);
+	spec->geneve_tlv_option_0_exist =
+		DEVX_GET(dr_match_set_misc, mask, geneve_tlv_option_0_exist);
 
 	spec->outer_ipv6_flow_label =
-		DEVX_GET(dr_match_set_misc, mask, outer_ipv6_flow_label);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, outer_ipv6_flow_label, clear);
 
 	spec->inner_ipv6_flow_label =
-		DEVX_GET(dr_match_set_misc, mask, inner_ipv6_flow_label);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, inner_ipv6_flow_label, clear);
 
-	spec->geneve_opt_len = DEVX_GET(dr_match_set_misc, mask, geneve_opt_len);
+	spec->geneve_opt_len = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, geneve_opt_len, clear);
 	spec->geneve_protocol_type =
-		DEVX_GET(dr_match_set_misc, mask, geneve_protocol_type);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, geneve_protocol_type, clear);
 
-	spec->bth_dst_qp = DEVX_GET(dr_match_set_misc, mask, bth_dst_qp);
+	spec->bth_dst_qp = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, bth_dst_qp, clear);
 }
 
-static void dr_ste_copy_mask_spec(char *mask, struct dr_match_spec *spec)
+static void dr_ste_copy_mask_spec(char *mask, struct dr_match_spec *spec, bool clear)
 {
-	spec->smac_47_16 = DEVX_GET(dr_match_spec, mask, smac_47_16);
+	spec->smac_47_16 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, smac_47_16, clear);
 
-	spec->smac_15_0 = DEVX_GET(dr_match_spec, mask, smac_15_0);
-	spec->ethertype = DEVX_GET(dr_match_spec, mask, ethertype);
+	spec->smac_15_0 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, smac_15_0, clear);
+	spec->ethertype = DR_DEVX_GET_CLEAR(dr_match_spec, mask, ethertype, clear);
 
-	spec->dmac_47_16 = DEVX_GET(dr_match_spec, mask, dmac_47_16);
+	spec->dmac_47_16 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, dmac_47_16, clear);
 
-	spec->dmac_15_0 = DEVX_GET(dr_match_spec, mask, dmac_15_0);
-	spec->first_prio = DEVX_GET(dr_match_spec, mask, first_prio);
-	spec->first_cfi = DEVX_GET(dr_match_spec, mask, first_cfi);
-	spec->first_vid = DEVX_GET(dr_match_spec, mask, first_vid);
+	spec->dmac_15_0 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, dmac_15_0, clear);
+	spec->first_prio = DR_DEVX_GET_CLEAR(dr_match_spec, mask, first_prio, clear);
+	spec->first_cfi = DR_DEVX_GET_CLEAR(dr_match_spec, mask, first_cfi, clear);
+	spec->first_vid = DR_DEVX_GET_CLEAR(dr_match_spec, mask, first_vid, clear);
 
-	spec->ip_protocol = DEVX_GET(dr_match_spec, mask, ip_protocol);
-	spec->ip_dscp = DEVX_GET(dr_match_spec, mask, ip_dscp);
-	spec->ip_ecn = DEVX_GET(dr_match_spec, mask, ip_ecn);
-	spec->cvlan_tag = DEVX_GET(dr_match_spec, mask, cvlan_tag);
-	spec->svlan_tag = DEVX_GET(dr_match_spec, mask, svlan_tag);
-	spec->frag = DEVX_GET(dr_match_spec, mask, frag);
-	spec->ip_version = DEVX_GET(dr_match_spec, mask, ip_version);
-	spec->tcp_flags = DEVX_GET(dr_match_spec, mask, tcp_flags);
-	spec->tcp_sport = DEVX_GET(dr_match_spec, mask, tcp_sport);
-	spec->tcp_dport = DEVX_GET(dr_match_spec, mask, tcp_dport);
+	spec->ip_protocol = DR_DEVX_GET_CLEAR(dr_match_spec, mask, ip_protocol, clear);
+	spec->ip_dscp = DR_DEVX_GET_CLEAR(dr_match_spec, mask, ip_dscp, clear);
+	spec->ip_ecn = DR_DEVX_GET_CLEAR(dr_match_spec, mask, ip_ecn, clear);
+	spec->cvlan_tag = DR_DEVX_GET_CLEAR(dr_match_spec, mask, cvlan_tag, clear);
+	spec->svlan_tag = DR_DEVX_GET_CLEAR(dr_match_spec, mask, svlan_tag, clear);
+	spec->frag = DR_DEVX_GET_CLEAR(dr_match_spec, mask, frag, clear);
+	spec->ip_version = DR_DEVX_GET_CLEAR(dr_match_spec, mask, ip_version, clear);
+	spec->tcp_flags = DR_DEVX_GET_CLEAR(dr_match_spec, mask, tcp_flags, clear);
+	spec->tcp_sport = DR_DEVX_GET_CLEAR(dr_match_spec, mask, tcp_sport, clear);
+	spec->tcp_dport = DR_DEVX_GET_CLEAR(dr_match_spec, mask, tcp_dport, clear);
 
-	spec->ipv4_ihl = DEVX_GET(dr_match_spec, mask, ipv4_ihl);
-	spec->l3_ok = DEVX_GET(dr_match_spec, mask, l3_ok);
-	spec->l4_ok = DEVX_GET(dr_match_spec, mask, l4_ok);
-	spec->ipv4_checksum_ok = DEVX_GET(dr_match_spec, mask, ipv4_checksum_ok);
-	spec->l4_checksum_ok = DEVX_GET(dr_match_spec, mask, l4_checksum_ok);
-	spec->ip_ttl_hoplimit = DEVX_GET(dr_match_spec, mask, ip_ttl_hoplimit);
+	spec->ipv4_ihl = DR_DEVX_GET_CLEAR(dr_match_spec, mask, ipv4_ihl, clear);
+	spec->l3_ok = DR_DEVX_GET_CLEAR(dr_match_spec, mask, l3_ok, clear);
+	spec->l4_ok = DR_DEVX_GET_CLEAR(dr_match_spec, mask, l4_ok, clear);
+	spec->ipv4_checksum_ok = DR_DEVX_GET_CLEAR(dr_match_spec, mask, ipv4_checksum_ok, clear);
+	spec->l4_checksum_ok = DR_DEVX_GET_CLEAR(dr_match_spec, mask, l4_checksum_ok, clear);
+	spec->ip_ttl_hoplimit = DR_DEVX_GET_CLEAR(dr_match_spec, mask, ip_ttl_hoplimit, clear);
 
-	spec->udp_sport = DEVX_GET(dr_match_spec, mask, udp_sport);
-	spec->udp_dport = DEVX_GET(dr_match_spec, mask, udp_dport);
+	spec->udp_sport = DR_DEVX_GET_CLEAR(dr_match_spec, mask, udp_sport, clear);
+	spec->udp_dport = DR_DEVX_GET_CLEAR(dr_match_spec, mask, udp_dport, clear);
 
-	spec->src_ip_127_96 = DEVX_GET(dr_match_spec, mask, src_ip_127_96);
+	spec->src_ip_127_96 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, src_ip_127_96, clear);
 
-	spec->src_ip_95_64 = DEVX_GET(dr_match_spec, mask, src_ip_95_64);
+	spec->src_ip_95_64 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, src_ip_95_64, clear);
 
-	spec->src_ip_63_32 = DEVX_GET(dr_match_spec, mask, src_ip_63_32);
+	spec->src_ip_63_32 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, src_ip_63_32, clear);
 
-	spec->src_ip_31_0 = DEVX_GET(dr_match_spec, mask, src_ip_31_0);
+	spec->src_ip_31_0 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, src_ip_31_0, clear);
 
-	spec->dst_ip_127_96 = DEVX_GET(dr_match_spec, mask, dst_ip_127_96);
+	spec->dst_ip_127_96 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, dst_ip_127_96, clear);
 
-	spec->dst_ip_95_64 = DEVX_GET(dr_match_spec, mask, dst_ip_95_64);
+	spec->dst_ip_95_64 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, dst_ip_95_64, clear);
 
-	spec->dst_ip_63_32 = DEVX_GET(dr_match_spec, mask, dst_ip_63_32);
+	spec->dst_ip_63_32 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, dst_ip_63_32, clear);
 
-	spec->dst_ip_31_0 = DEVX_GET(dr_match_spec, mask, dst_ip_31_0);
+	spec->dst_ip_31_0 = DR_DEVX_GET_CLEAR(dr_match_spec, mask, dst_ip_31_0, clear);
 }
 
-static void dr_ste_copy_mask_misc2(char *mask, struct dr_match_misc2 *spec)
+static void dr_ste_copy_mask_misc2(char *mask, struct dr_match_misc2 *spec, bool clear)
 {
 	spec->outer_first_mpls_label =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_label);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_label, clear);
 	spec->outer_first_mpls_exp =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_exp);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_exp, clear);
 	spec->outer_first_mpls_s_bos =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_s_bos);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_s_bos, clear);
 	spec->outer_first_mpls_ttl =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_ttl);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_ttl, clear);
 	spec->inner_first_mpls_label =
-		DEVX_GET(dr_match_set_misc2, mask, inner_first_mpls_label);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, inner_first_mpls_label, clear);
 	spec->inner_first_mpls_exp =
-		DEVX_GET(dr_match_set_misc2, mask, inner_first_mpls_exp);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, inner_first_mpls_exp, clear);
 	spec->inner_first_mpls_s_bos =
-		DEVX_GET(dr_match_set_misc2, mask, inner_first_mpls_s_bos);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, inner_first_mpls_s_bos, clear);
 	spec->inner_first_mpls_ttl =
-		DEVX_GET(dr_match_set_misc2, mask, inner_first_mpls_ttl);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, inner_first_mpls_ttl, clear);
 	spec->outer_first_mpls_over_gre_label =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_over_gre_label);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_over_gre_label, clear);
 	spec->outer_first_mpls_over_gre_exp =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_over_gre_exp);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_over_gre_exp, clear);
 	spec->outer_first_mpls_over_gre_s_bos =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_over_gre_s_bos);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_over_gre_s_bos, clear);
 	spec->outer_first_mpls_over_gre_ttl =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_over_gre_ttl);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_over_gre_ttl, clear);
 	spec->outer_first_mpls_over_udp_label =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_over_udp_label);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_over_udp_label, clear);
 	spec->outer_first_mpls_over_udp_exp =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_over_udp_exp);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_over_udp_exp, clear);
 	spec->outer_first_mpls_over_udp_s_bos =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_over_udp_s_bos);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_over_udp_s_bos, clear);
 	spec->outer_first_mpls_over_udp_ttl =
-		DEVX_GET(dr_match_set_misc2, mask, outer_first_mpls_over_udp_ttl);
-	spec->metadata_reg_c_7 = DEVX_GET(dr_match_set_misc2, mask, metadata_reg_c_7);
-	spec->metadata_reg_c_6 = DEVX_GET(dr_match_set_misc2, mask, metadata_reg_c_6);
-	spec->metadata_reg_c_5 = DEVX_GET(dr_match_set_misc2, mask, metadata_reg_c_5);
-	spec->metadata_reg_c_4 = DEVX_GET(dr_match_set_misc2, mask, metadata_reg_c_4);
-	spec->metadata_reg_c_3 = DEVX_GET(dr_match_set_misc2, mask, metadata_reg_c_3);
-	spec->metadata_reg_c_2 = DEVX_GET(dr_match_set_misc2, mask, metadata_reg_c_2);
-	spec->metadata_reg_c_1 = DEVX_GET(dr_match_set_misc2, mask, metadata_reg_c_1);
-	spec->metadata_reg_c_0 = DEVX_GET(dr_match_set_misc2, mask, metadata_reg_c_0);
-	spec->metadata_reg_a = DEVX_GET(dr_match_set_misc2, mask, metadata_reg_a);
-	spec->metadata_reg_b = DEVX_GET(dr_match_set_misc2, mask, metadata_reg_b);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, outer_first_mpls_over_udp_ttl, clear);
+	spec->metadata_reg_c_7 = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_c_7, clear);
+	spec->metadata_reg_c_6 = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_c_6, clear);
+	spec->metadata_reg_c_5 = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_c_5, clear);
+	spec->metadata_reg_c_4 = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_c_4, clear);
+	spec->metadata_reg_c_3 = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_c_3, clear);
+	spec->metadata_reg_c_2 = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_c_2, clear);
+	spec->metadata_reg_c_1 = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_c_1, clear);
+	spec->metadata_reg_c_0 = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_c_0, clear);
+	spec->metadata_reg_a = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_a, clear);
 }
 
-static void dr_ste_copy_mask_misc3(char *mask, struct dr_match_misc3 *spec)
+static void dr_ste_copy_mask_misc3(char *mask, struct dr_match_misc3 *spec, bool clear)
 {
-	spec->inner_tcp_seq_num = DEVX_GET(dr_match_set_misc3, mask, inner_tcp_seq_num);
-	spec->outer_tcp_seq_num = DEVX_GET(dr_match_set_misc3, mask, outer_tcp_seq_num);
-	spec->inner_tcp_ack_num = DEVX_GET(dr_match_set_misc3, mask, inner_tcp_ack_num);
-	spec->outer_tcp_ack_num = DEVX_GET(dr_match_set_misc3, mask, outer_tcp_ack_num);
+	spec->inner_tcp_seq_num = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, inner_tcp_seq_num, clear);
+	spec->outer_tcp_seq_num = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, outer_tcp_seq_num, clear);
+	spec->inner_tcp_ack_num = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, inner_tcp_ack_num, clear);
+	spec->outer_tcp_ack_num = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, outer_tcp_ack_num, clear);
 	spec->outer_vxlan_gpe_vni =
-		DEVX_GET(dr_match_set_misc3, mask, outer_vxlan_gpe_vni);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, outer_vxlan_gpe_vni, clear);
 	spec->outer_vxlan_gpe_next_protocol =
-		DEVX_GET(dr_match_set_misc3, mask, outer_vxlan_gpe_next_protocol);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, outer_vxlan_gpe_next_protocol, clear);
 	spec->outer_vxlan_gpe_flags =
-		DEVX_GET(dr_match_set_misc3, mask, outer_vxlan_gpe_flags);
-	spec->icmpv4_header_data = DEVX_GET(dr_match_set_misc3, mask, icmp_header_data);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, outer_vxlan_gpe_flags, clear);
+	spec->icmpv4_header_data = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, icmp_header_data, clear);
 	spec->icmpv6_header_data =
-		DEVX_GET(dr_match_set_misc3, mask, icmpv6_header_data);
-	spec->icmpv4_type = DEVX_GET(dr_match_set_misc3, mask, icmp_type);
-	spec->icmpv4_code = DEVX_GET(dr_match_set_misc3, mask, icmp_code);
-	spec->icmpv6_type = DEVX_GET(dr_match_set_misc3, mask, icmpv6_type);
-	spec->icmpv6_code = DEVX_GET(dr_match_set_misc3, mask, icmpv6_code);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, icmpv6_header_data, clear);
+	spec->icmpv4_type = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, icmp_type, clear);
+	spec->icmpv4_code = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, icmp_code, clear);
+	spec->icmpv6_type = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, icmpv6_type, clear);
+	spec->icmpv6_code = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, icmpv6_code, clear);
 	spec->geneve_tlv_option_0_data =
-		DEVX_GET(dr_match_set_misc3, mask, geneve_tlv_option_0_data);
-	spec->gtpu_msg_flags = DEVX_GET(dr_match_set_misc3, mask, gtpu_msg_flags);
-	spec->gtpu_msg_type = DEVX_GET(dr_match_set_misc3, mask, gtpu_msg_type);
-	spec->gtpu_teid = DEVX_GET(dr_match_set_misc3, mask, gtpu_teid);
-	spec->gtpu_dw_0 = DEVX_GET(dr_match_set_misc3, mask, gtpu_dw_0);
-	spec->gtpu_dw_2 = DEVX_GET(dr_match_set_misc3, mask, gtpu_dw_2);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, geneve_tlv_option_0_data, clear);
+	spec->gtpu_msg_flags = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_msg_flags, clear);
+	spec->gtpu_msg_type = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_msg_type, clear);
+	spec->gtpu_teid = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_teid, clear);
+	spec->gtpu_dw_0 = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_dw_0, clear);
+	spec->gtpu_dw_2 = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_dw_2, clear);
 	spec->gtpu_first_ext_dw_0 =
-		DEVX_GET(dr_match_set_misc3, mask, gtpu_first_ext_dw_0);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_first_ext_dw_0, clear);
 }
 
-static void dr_ste_copy_mask_misc4(char *mask, struct dr_match_misc4 *spec)
+static void dr_ste_copy_mask_misc4(char *mask, struct dr_match_misc4 *spec, bool clear)
 {
 	spec->prog_sample_field_id_0 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_id_0);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_id_0, clear);
 	spec->prog_sample_field_value_0 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_value_0);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_value_0, clear);
 	spec->prog_sample_field_id_1 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_id_1);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_id_1, clear);
 	spec->prog_sample_field_value_1 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_value_1);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_value_1, clear);
 	spec->prog_sample_field_id_2 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_id_2);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_id_2, clear);
 	spec->prog_sample_field_value_2 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_value_2);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_value_2, clear);
 	spec->prog_sample_field_id_3 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_id_3);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_id_3, clear);
 	spec->prog_sample_field_value_3 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_value_3);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_value_3, clear);
 	spec->prog_sample_field_id_4 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_id_4);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_id_4, clear);
 	spec->prog_sample_field_value_4 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_value_4);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_value_4, clear);
 	spec->prog_sample_field_id_5 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_id_5);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_id_5, clear);
 	spec->prog_sample_field_value_5 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_value_5);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_value_5, clear);
 	spec->prog_sample_field_id_6 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_id_6);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_id_6, clear);
 	spec->prog_sample_field_value_6 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_value_6);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_value_6, clear);
 	spec->prog_sample_field_id_7 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_id_7);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_id_7, clear);
 	spec->prog_sample_field_value_7 =
-		DEVX_GET(dr_match_set_misc4, mask, prog_sample_field_value_7);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc4, mask, prog_sample_field_value_7, clear);
 }
 
-static void dr_ste_copy_mask_misc5(char *mask, struct dr_match_misc5 *spec)
+static void dr_ste_copy_mask_misc5(char *mask, struct dr_match_misc5 *spec, bool clear)
 {
 	spec->macsec_tag_0 =
-		DEVX_GET(dr_match_set_misc5, mask, macsec_tag_0);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc5, mask, macsec_tag_0, clear);
 	spec->macsec_tag_1 =
-		DEVX_GET(dr_match_set_misc5, mask, macsec_tag_1);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc5, mask, macsec_tag_1, clear);
 	spec->macsec_tag_2 =
-		DEVX_GET(dr_match_set_misc5, mask, macsec_tag_2);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc5, mask, macsec_tag_2, clear);
 	spec->macsec_tag_3 =
-		DEVX_GET(dr_match_set_misc5, mask, macsec_tag_3);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc5, mask, macsec_tag_3, clear);
 	spec->tunnel_header_0 =
-		DEVX_GET(dr_match_set_misc5, mask, tunnel_header_0);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc5, mask, tunnel_header_0, clear);
 	spec->tunnel_header_1 =
-		DEVX_GET(dr_match_set_misc5, mask, tunnel_header_1);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc5, mask, tunnel_header_1, clear);
 	spec->tunnel_header_2 =
-		DEVX_GET(dr_match_set_misc5, mask, tunnel_header_2);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc5, mask, tunnel_header_2, clear);
 	spec->tunnel_header_3 =
-		DEVX_GET(dr_match_set_misc5, mask, tunnel_header_3);
+		DR_DEVX_GET_CLEAR(dr_match_set_misc5, mask, tunnel_header_3, clear);
 }
 
-#define MAX_PARAM_SIZE 512
-
 void dr_ste_copy_param(uint8_t match_criteria,
 		       struct dr_match_param *set_param,
-		       struct mlx5dv_flow_match_parameters *mask)
+		       uint64_t *mask_buf, size_t mask_sz, bool clear)
 {
-	char tail_param[MAX_PARAM_SIZE] = {};
+	uint8_t *data = (uint8_t *)mask_buf;
+	char tail_param[DEVX_ST_SZ_BYTES(dr_match_param)] = {};
 	size_t param_location;
-	uint8_t *data = (uint8_t *)mask->match_buf;
 	void *buff;
 
 	if (match_criteria & DR_MATCHER_CRITERIA_OUTER) {
-		if (mask->match_sz < DEVX_ST_SZ_BYTES(dr_match_spec)) {
-			memcpy(tail_param, data, mask->match_sz);
+		if (mask_sz < DEVX_ST_SZ_BYTES(dr_match_spec)) {
+			memcpy(tail_param, data, mask_sz);
 			buff = tail_param;
 		} else {
-			buff = mask->match_buf;
+			buff = mask_buf;
 		}
-		dr_ste_copy_mask_spec(buff, &set_param->outer);
+		dr_ste_copy_mask_spec(buff, &set_param->outer, clear);
 	}
 	param_location = DEVX_ST_SZ_BYTES(dr_match_spec);
 
 	if (match_criteria & DR_MATCHER_CRITERIA_MISC) {
-		if (mask->match_sz < param_location +
+		if (mask_sz < param_location +
 		    DEVX_ST_SZ_BYTES(dr_match_set_misc)) {
 			memcpy(tail_param, data + param_location,
-			       mask->match_sz - param_location);
+			       mask_sz - param_location);
 			buff = tail_param;
 		} else {
 			buff = data + param_location;
 		}
-		dr_ste_copy_mask_misc(buff, &set_param->misc);
+		dr_ste_copy_mask_misc(buff, &set_param->misc, clear);
 	}
 	param_location += DEVX_ST_SZ_BYTES(dr_match_set_misc);
 
 	if (match_criteria & DR_MATCHER_CRITERIA_INNER) {
-		if (mask->match_sz < param_location +
+		if (mask_sz < param_location +
 		    DEVX_ST_SZ_BYTES(dr_match_spec)) {
 			memcpy(tail_param, data + param_location,
-			       mask->match_sz - param_location);
+			       mask_sz - param_location);
 			buff = tail_param;
 		} else {
 			buff = data + param_location;
 		}
-		dr_ste_copy_mask_spec(buff, &set_param->inner);
+		dr_ste_copy_mask_spec(buff, &set_param->inner, clear);
 	}
 	param_location += DEVX_ST_SZ_BYTES(dr_match_spec);
 
 	if (match_criteria & DR_MATCHER_CRITERIA_MISC2) {
-		if (mask->match_sz < param_location +
+		if (mask_sz < param_location +
 		    DEVX_ST_SZ_BYTES(dr_match_set_misc2)) {
 			memcpy(tail_param, data + param_location,
-			       mask->match_sz - param_location);
+			       mask_sz - param_location);
 			buff = tail_param;
 		} else {
 			buff = data + param_location;
 		}
-		dr_ste_copy_mask_misc2(buff, &set_param->misc2);
+		dr_ste_copy_mask_misc2(buff, &set_param->misc2, clear);
 	}
 	param_location += DEVX_ST_SZ_BYTES(dr_match_set_misc2);
 
 	if (match_criteria & DR_MATCHER_CRITERIA_MISC3) {
-		if (mask->match_sz < param_location +
+		if (mask_sz < param_location +
 		    DEVX_ST_SZ_BYTES(dr_match_set_misc3)) {
 			memcpy(tail_param, data + param_location,
-			       mask->match_sz - param_location);
+			       mask_sz - param_location);
 			buff = tail_param;
 		} else {
 			buff = data + param_location;
 		}
-		dr_ste_copy_mask_misc3(buff, &set_param->misc3);
+		dr_ste_copy_mask_misc3(buff, &set_param->misc3, clear);
 	}
 	param_location += DEVX_ST_SZ_BYTES(dr_match_set_misc3);
 
 	if (match_criteria & DR_MATCHER_CRITERIA_MISC4) {
-		if (mask->match_sz < param_location +
+		if (mask_sz < param_location +
 		    DEVX_ST_SZ_BYTES(dr_match_set_misc4)) {
 			memcpy(tail_param, data + param_location,
-			       mask->match_sz - param_location);
+			       mask_sz - param_location);
 			buff = tail_param;
 		} else {
 			buff = data + param_location;
 		}
-		dr_ste_copy_mask_misc4(buff, &set_param->misc4);
+		dr_ste_copy_mask_misc4(buff, &set_param->misc4, clear);
 	}
 	param_location += DEVX_ST_SZ_BYTES(dr_match_set_misc4);
 
 	if (match_criteria & DR_MATCHER_CRITERIA_MISC5) {
-		if (mask->match_sz < param_location +
+		if (mask_sz < param_location +
 		    DEVX_ST_SZ_BYTES(dr_match_set_misc5)) {
 			memcpy(tail_param, data + param_location,
-			       mask->match_sz - param_location);
+			       mask_sz - param_location);
 			buff = tail_param;
 		} else {
 			buff = data + param_location;
 		}
-		dr_ste_copy_mask_misc5(buff, &set_param->misc5);
+		dr_ste_copy_mask_misc5(buff, &set_param->misc5, clear);
 	}
 }
 
@@ -1243,6 +1300,21 @@ void dr_ste_build_tnl_mpls_over_udp(struct dr_ste_ctx *ste_ctx,
 	ste_ctx->build_tnl_mpls_over_udp_init(sb, mask);
 }
 
+void dr_ste_build_tnl_geneve_tlv_opt_exist(struct dr_ste_ctx *ste_ctx,
+					   struct dr_ste_build *sb,
+					   struct dr_match_param *mask,
+					   struct dr_devx_caps *caps,
+					   bool inner, bool rx)
+{
+	if (!ste_ctx->build_tnl_geneve_tlv_opt_exist_init)
+		return;
+
+	sb->rx = rx;
+	sb->inner = inner;
+	sb->caps = caps;
+	ste_ctx->build_tnl_geneve_tlv_opt_exist_init(sb, mask);
+}
+
 void dr_ste_build_icmp(struct dr_ste_ctx *ste_ctx,
 		       struct dr_ste_build *sb,
 		       struct dr_match_param *mask,
@@ -1393,14 +1465,17 @@ void dr_ste_build_flex_parser_1(struct dr_ste_ctx *ste_ctx,
 	ste_ctx->build_flex_parser_1_init(sb, mask);
 }
 
-void dr_ste_build_tunnel_header_0_1(struct dr_ste_ctx *ste_ctx,
-				    struct dr_ste_build *sb,
-				    struct dr_match_param *mask,
-				    bool inner, bool rx)
+void dr_ste_build_tunnel_header(struct dr_ste_ctx *ste_ctx,
+				struct dr_ste_build *sb,
+				struct dr_match_param *mask,
+				struct dr_devx_caps *caps,
+				bool inner, bool rx)
 {
 	sb->rx = rx;
 	sb->inner = inner;
-	ste_ctx->build_tunnel_header_0_1(sb, mask);
+	sb->caps = caps;
+
+	ste_ctx->build_tunnel_header_init(sb, mask);
 }
 
 int dr_ste_build_def0(struct dr_ste_ctx *ste_ctx,
@@ -1585,6 +1660,8 @@ struct dr_ste_ctx *dr_ste_get_ctx(uint8_t version)
 		return dr_ste_get_ctx_v0();
 	else if (version == MLX5_HW_CONNECTX_6DX)
 		return dr_ste_get_ctx_v1();
+	else if (version == MLX5_HW_CONNECTX_7)
+		return dr_ste_get_ctx_v2();
 
 	errno = EOPNOTSUPP;
 
