@@ -1,11 +1,12 @@
 # SPDX-License-Identifier: (GPL-2.0 OR Linux-OpenIB)
-# Copyright 2020 Amazon.com, Inc. or its affiliates. All rights reserved.
+# Copyright 2020-2022 Amazon.com, Inc. or its affiliates. All rights reserved.
 
 import unittest
 import random
 import errno
 
 from pyverbs.pyverbs_error import PyverbsRDMAError
+from pyverbs.cq import CqInitAttrEx
 from pyverbs.qp import QPAttr, QPCap, QPInitAttrEx
 import pyverbs.providers.efa.efa_enums as efa_e
 import pyverbs.providers.efa.efadv as efa
@@ -96,3 +97,35 @@ class SRDResources(TrafficResources):
             self.mr = tests.utils.create_custom_mr(self, e.IBV_ACCESS_REMOTE_READ)
         else:
             self.mr = tests.utils.create_custom_mr(self)
+
+
+class EfaCQRes(SRDResources):
+    def __init__(self, dev_name, ib_port, gid_index, send_ops_flags,
+                 qp_count=1, requested_dev_cap=None, wc_flags=None):
+        """
+        Initialize EFA DV CQ based on SRD resources.
+        :param requested_dev_cap: A necessary device cap. If it's not supported
+                                  by the device, the test will be skipped.
+        :param wc_flags: WC flags for EFA DV CQ.
+        """
+        self.requested_dev_cap = requested_dev_cap
+        self.efa_wc_flags = wc_flags
+        super().__init__(dev_name, ib_port, gid_index, send_ops_flags, qp_count=qp_count)
+
+    def create_context(self):
+        super().create_context()
+        if self.requested_dev_cap:
+            with efa.EfaContext(name=self.ctx.name) as efa_ctx:
+                if not efa_ctx.query_efa_device().device_caps & self.requested_dev_cap:
+                    miss_caps = efa.dev_cap_to_str(self.requested_dev_cap)
+                    raise unittest.SkipTest(f'Device caps doesn\'t support {miss_caps}')
+
+    def create_cq(self):
+        cia = CqInitAttrEx(wc_flags=e.IBV_WC_STANDARD_FLAGS)
+        efa_cia = efa.EfaDVCQInitAttr(self.efa_wc_flags)
+        try:
+            self.cq = efa.EfaCQ(self.ctx, cia, efa_cia)
+        except PyverbsRDMAError as ex:
+            if ex.error_code == errno.EOPNOTSUPP:
+                raise unittest.SkipTest('Create EFA DV CQ is not supported')
+            raise ex
